#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Image et doublons
# 
# Material for the hackathon ENSAE / BRGM / 2018. Les images sont extraites de tweets mais sont retweetées sans être retweetées.

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt


# In[2]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Séparation des doublons
# 
# Pour le challenge, il faut repérer les doublons dans les images. Pour cela, je zoom chaque image sur un carré 50x50 en noir et blanc, suivi d'une ACP puis k plus proches voisins pour détecter les doublons.

# ### Images en gris 50x50

# In[3]:


folder = "c:/temp/suricatenat_images"


# In[4]:


from ensae_projects.hackathon.image_helper import apply_image_transform, image_zoom, img2gray

dest_folder = "img5050"
list(apply_image_transform(folder, dest_folder, lambda img: image_zoom(img2gray(img), (50, 50)), fLOG=print))


# ### Images en features
# 
# Pas utilisé par la suite.

# In[5]:


from ensae_projects.hackathon.image_helper import stream_image2features
import numpy

dest_folder = "img5050"
dest_batch = "batch"
for b in stream_image2features(dest_folder, dest_batch, numpy.array, fLOG=print):
    pass


# ### voisins

# In[6]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[7]:


from ensae_projects.hackathon.image_knn import ImageNearestNeighbors
folder = "img5050"
knn = ImageNearestNeighbors()
knn.fit(folder, fLOG=print)


# In[8]:


from ensae_projects.hackathon.image_helper import enumerate_image_class
folder = "img5050"
iter = enumerate_image_class(folder)
imgs = [_[0] for _ in zip(iter, range(0,1000000))]
len(imgs)


# In[9]:


for i, img in enumerate(imgs):
    dist, ind = knn.kneighbors(img[0])
    if dist[0, 1] <= 10:
        print("dist =", dist)
        print("ind =", ind)
        break


# In[10]:


knn.plot_neighbors(ind, dist, obs=img[0], folder_or_images=folder);


# In[11]:


pairs = []
for i, img in enumerate(imgs):
    if i % 1000 == 0:
        print("{0}/{1} done".format(i, len(imgs)))
    dist, ind = knn.kneighbors(img[0])
    sub = ind.ravel()[dist.ravel() <= 10]
    if len(sub) > 0:
        for j in sub:
            pairs.append((i, j))


# In[12]:


pairs[:10]


# In[13]:


pairs2 = [(i,j) for i,j in pairs if i != j]
len(pairs), len(pairs2)


# In[14]:


pairs2[:10]


# In[15]:


dist, ind = knn.kneighbors(imgs[5][0])
knn.plot_neighbors(ind, dist, obs=imgs[5][0], folder_or_images=folder);


# ### Composantes connectes

# In[16]:


distincts = []
for i, j in pairs2:
    distincts.append(i)
    distincts.append(j)
distincts = set(distincts)
connex = {}
for k in distincts:
    connex[k] = k

n = 0
while n < 10:
    modif = 0
    for i, j in pairs2:        
        a = min(connex[i], connex[j])
        if a != connex[i] or a != connex[j]:
            modif += 1
        connex[i] = connex[j] = a
    print(n, modif)
    n += 1


# In[17]:


len(connex), len(set(connex.values()))


# In[18]:


names = knn.image_names_
names[:2]


# In[19]:


dups = []
for i, j in connex.items():
    if i != j:
        dups.append(names[i])
len(dups)


# ### Images très proches

# In[20]:


for i, img in enumerate(imgs):
    dist, ind = knn.kneighbors(img[0])
    if 10 < dist[0, 1] <= 30:
        print("dist =", dist)
        print("ind =", ind)
        break


# In[21]:


obs = imgs[ind[0, 0]][0]
knn.plot_neighbors(ind, dist, obs=obs, folder_or_images=folder);


# ### Recopie de la base

# In[22]:


not_allowed = set(dups)
len(not_allowed)


# In[23]:


list(sorted(not_allowed))[:5]


# In[24]:


from ensae_projects.hackathon.image_helper import stream_copy_images

src_folder = "c:/temp/suricatenat_images/"
dest_folder = "c:/temp/suricatenat_clean/"

def valid(name):
    spl = name.split("suricatenat_images")[-1].replace("\\", "/").strip("/\\")
    return spl not in allowed

for img in stream_copy_images(src_folder, dest_folder, valid, fLOG=print):
    pass


# In[25]:


l1 = list(enumerate_image_class("c:/temp/suricatenat_images/"))


# In[26]:


l2 = list(enumerate_image_class("c:/temp/suricatenat_clean/"))


# In[27]:


len(l1), len(l2)


# ### Takes a random sample

# In[28]:


from ensae_projects.hackathon.image_helper import stream_random_sample, last_element
rnd = last_element(stream_random_sample("c:/temp/suricatenat_clean/", abspath=False))


# In[29]:


rnd[:5]


# In[30]:


import os
import shutil

src_folder = "c:/temp/suricatenat_clean/"
dest_folder = "c:/temp/suricatenat_sample/"

    
for img, sub in rnd:
    src = os.path.join(src_folder, img)
    dst = os.path.join(dest_folder, img)
    d = os.path.dirname(dst)
    if not os.path.exists(d):
        os.makedirs(d)
    
    shutil.copy(src, dst)


# In[31]: